#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## storiq-rescue.pm for storiq-rescue
## 
## ######### MODULE DESCRIPTION : ###
## 
## Fonctions permettantes de recuperer
## les differentes variables necessaires
## a la modification du fstab au boot.
## 
## ###################################
## 
## Made by Sardinha Mickael
## Login   <aboutonnet@intellique.com>
## 
## Started on  Tue May 30 12:36:47 2006 Sardinha Mickael
## Last update Fri Sep 26 16:06:38 2008 Boutonnet Alexandre
## 
## ###################################
##

## ###################################
## Declaration de librairies

package Storiq_Rescue;

use strict;
use warnings;
use Term::ANSIColor;
use Tie::File;
use Data::Dumper;

## ###################################
## Methode de recuperation du root en fonction
## du label
# Remonte la parition racine + ecriture
sub	do_rootpart
{
    #Remonte la partition racine en lecture + ecriture afin de
    #pouvoir faire les modifications au /etc/fstab
    eval {
	qx#mount -o remount /#;
    };
    if ($@) {
	return (1,$@);
    }
    return 0;
}


# Gestion des point de montage du fstab
sub	do_mountpoint
{

    if ( -w "/etc/fstab") {
	# Ouverture du fichier fstab
	open (FD, "/etc/fstab");
	
	# parcours du fichier
	foreach my $var (<FD>) {
	    
	    # On ignore les commentaires
	    next if ($var =~ /^\s*\#/) ;
	    
	    # On recupere les points de montage
	    if ($var =~ s/^\s*.*\s+(\/\S+).*/$1/ ) {
		
		chomp ($var);
		
		# On cree les points de montage si ils n'existent pas
		if ( ! -d $var){
		    mkdir ($var, 0755) or die "Error: $!\n";
		}
	    }
	}
	
	# Creation du point de montage /sys si il n'existe pas
	if ( ! -d "/sys") {
	    mkdir ("/sys", 0755) or die "Error : $! \n";
	}
    }
    else {
	return 1;
    }
    return 0;
}

# Creation des devices avec MAKEDEV generic
# CETTE FONCTION EST DEVENUE OBSOLETTE
# NOUS UTILISONS UDEV MAINTENANT
sub	do_dev
{
    if ( ! -e "/dev/zero") {
	# Si le repertoire /dev n'existe pas on le cree
	if ( ! -d "/dev/") {
	    mkdir ("/dev", 0755) or die "Error: $!.\n";
	}
	# Execution du script de creation des devices
	system 'cd /dev; /sbin/MAKEDEV generic';
    }   
}

# Gestion du /proc
sub	do_proc
{
    # Montage de /proc si /proc/cmdline n'existe pas
    if ( ! -f "/proc/cmdline" ) {
	eval {
	    qx#mount -t proc none /proc#;
	};
	if ($@) {
            return (1,$@);
	}	
	return 0;
    } 
    else {
	return 0;
    }
}

# Demonte /proc et supprime son point de montage
sub	undo_proc
{
    # On demonte /proc

    eval {
	qx#umount /proc#;
    };
    
    if ($@){
	return (1,$@);
    }
    return 0;
}


# Determination du mode de lancement
sub	get_cmdline
{

    # Ouverture en lecture du fichier cmdline
    open (FD, "/proc/cmdline") or die "Error: $!.\n";
    
    # On parcourt le fichier
    while (<FD>){
	
	# On ignore les commentaires
	next if ($_ =~ /^\s*\#/) ;
	
	# Recuperation du nom du mode de lancement
	if ( m/BOOT_IMAGE\s*=\s*([\w\.\-]+)\s*/ )
	{
	    close (FD);
	    return $1;
	}
    }

    # Fermeture du fichier
    close (FD);
    # Sinon erreur dans le fichier
    return undef;

}


# Recuperation du root actuel (l'ancien)
sub	get_oldroot
{
    # Ouverture en lecture du fstab
    open (FD, "/etc/fstab") or die "Error: $!.\n";
    
    # Parcours du fichier
    while (<FD>){

	# On ignore les commentaires
	next if ($_ =~ /^\s*\#/) ;

	# Regexp pour recuperer le nouveau root
	if ( m/^\s*(\/\w+\/\w+)\s+\/\s+.*/ ) {
	    close (FD);
	    return $1;
	}
    }
    
    # Fermeture du fichier
    close (FD);
    
    # Sinon c'est quil y a une erreur dans le fichier
    return undef;
}

# Modification du /etc/fstab 
sub	do_modif
{
    my $old = shift;
    my $new = shift;

    # Ouverture en lecture de la copie de fstab
    open (FD_READ, "< /etc/fstab.orig")
	or die "Error: $!.\n";

    # Ouverture en ecriture du nouveau fstab
    open (FD_WRITE, "> /etc/fstab")
	or die "Error: $!.\n";

    # On Parcourt le fichier
    while (<FD_READ>) {

	# Si dans le nouveau fstab on match l'ancien root
	# on le remplace par le nouveau
	if ( $_ =~ s/$old/$new/ ){
	    print FD_WRITE "$_";
	}

	# Sinon on remplace rien
	else {
	    print FD_WRITE "$_";
	}
    }

    # Fermetures des fichiers
    close (FD_READ);
    close (FD_WRITE);
}

# Recuperation du nouveau root par le lilo.conf
sub	get_root 
{

    my $entree		= shift;
    my $root_dft	= "";
    my $bloc_flag	= 0;
    my $current_root	= "";
    my $current_label	= "";

    # Correction du bug #290
    $entree = "rescue" if ($entree =~ m/rescue/);

    # Ouverture en lecture du fichier /etc/lilo.conf
    open (FD, "/etc/lilo.conf")
	or die "Error: $! \n";

    # Parcours du fichier lilo.conf
    foreach my $var (<FD>){
	
	# On ignore les commentaires
	next if ($var =~ /^\s*\#/) ;

	# Match un root 
	if ($var =~ s/^\s*root\s*=\s*(\/\w+\/\w+)\s*$/$1/ ){
	    
	    # Dans le cas ou le root matche n'est pas dans un bloc
	    # on positionne le root par defaut
	    if (!$bloc_flag){
		$root_dft = $var;
	    }
	    # Dans le cas ou le root matche est dans un bloc image
	    # on positionne le root courant
	    else {
		$current_root = $var;
	    }
	}

	# Match un label et on positionne la variable label
	elsif ($var =~ s/^\s*label\s*=\s*(.*)\s*$/$1/) {
	    $current_label = $var;
	}

	# Match une image ou un other
	elsif ($var =~ m/^\s*(image|other)\s*=/ ) {

	    # On positionne un flag a 1 quand on rencontre le 1er bloc image ou other 
	    if (!$bloc_flag){
		$bloc_flag = 1;
	    }
	    
	    # Si le label est celui desire
	    if($current_label && ($current_label eq $entree)){

		# Si la variable du root courant n'est pas vide, on l'a retourne
		if ($current_root){
		    close(FD);
	    	    return $current_root;
		}

		# Ou si le root par defaut n'est pas vide, on le retourne
		elsif ($root_dft) {
		    close(FD);
		    return $root_dft;
		}
		# Sinon on retourne une erreur
		else {
		    close(FD);
		    return undef;
		}

	    }

	    # Vide les variables root courant et label, et pass a la ligne suivante si le label est vide et/ou
	    # ne correspond pas au label desire
	    else {
		$current_root	= "";
		$current_label	= "";
		next;
	    }
	}
    }

    #########################################################################
    ## Verificaton du root courant et de label a la sorti de la boucle 
    ## si le root courant et label n'ont pas encore ete positionne

    # Si le label est celui desire
    if($current_label && ($current_label eq $entree)){

        # Si la variable du root courant n'est pas vide on la retourne
	if ($current_root){
	    close(FD);
	    return $current_root;
	}

        # Ou Si le root par defaut n'est  pas vide, on le retourne
	elsif ($root_dft){
	    close(FD);
	    return $root_dft;
	}
	# Sinon on retourne une erreur
	else {
	    close(FD);
	    return undef;
	}
    }

    # Si le label desire n'a pas ete matcher, retourne une erreur
    else {
	close(FD);
	return undef;
    }
    
    # Fermeture du fichier /etc/lilo.conf
    close(FD);

    return undef;
}

# Cette fonction commente la ligne /mnt/rescue
# Cette fonction corrige le bug #287
sub unable_rescue
{
    my ($root) = @_;
    
    # Verification et ouverture du fichier
    my @file;
    
    unless (tie (@file, 'Tie::File', "/etc/fstab"))
    {
        return (1, "Unable to open file : /etc/fstab");
    }
    
    my $pos = 0;
    while ($pos < scalar(@file))
    {
	my $line = $file[$pos];
	
	if ($line =~ m/^.*$root\s+\/\w+.*$/)
	{
	    if ($line !~ m/^\#.*/)
	    {
		splice (@file, $pos++, 0, "# Commented out by Storiq rescue script");
		$file[$pos] = '# '.$line;
	    }
	}
	$pos++;
    }
    
    # Close le fichier
    untie (@file);
    
    return (0, 0);
}

1;
